;Music-data specification for the player
;These values are not exactly what are stored in memory, but they can convey the information
;about the usable value-ranges of different data-categories better. If there was an editor it should
;display the values this way and convert between displayed values to their memory-representations
;on the fly like FlexSID does. (The real values are in the shifted form to support smaller playercode.)
;
;Track: $00..$3F:Jump (0..63)
;       $40..$5F:Instrument, $60..$7F:Transpose, if only one of them supported: $40..$7F: Ins./Transp.
;       $80..$FF:Pattern (translated to 0..127)
;
;Pattern: $00..$1F: Note (tableindex 0..31)
;         $20..$3F: Note (tableindex 0..31) + Pattern-END
;         $40..$7F: Instrument (index 0..63)
;         $80..$FF: Duration-change (0..127, note-rowcount multiplied by tempo)
;
;Instrument: $00..$0F:Arp(rel.pitch)+WF, $10..$1F:Abs.pitch+WF, if no abs pitch, $00..$1F:Arp+WF
;            $20..$3F: SID-setting (channel-index added), maybe later $20..$2F:Arp without WF, $30..$3F:SID
;            $40..$7F: The same Arpeggio/SIDsetting as $00..$3F, but with End (instrument-table stops)
;            $80..$FF: Jump (0..127) in instrument-region


;Convenience macro-functions (specification) for music-data

.if (TRANSPOSE_SUPPORT || TRACKINS_SUPPORT)
 JMP .sfunction index, ( (index-TRACKS) << 1 )        |    %0 ;Jump in Track
 .if (TRANSPOSE_SUPPORT && TRACKINS_SUPPORT)
  INS .sfunction index, ( (index+1-INSTRUMENTS) << 2 ) |   %01 ;Instrument in Track
  TRA .sfunction tran, ( tran << 2 )                   |   %11 ;Transpose in Track
 .elsif (TRANSPOSE_SUPPORT)
  TRA .sfunction tran, ( tran << 1 )                   |    %1 ;Transpose in Track
 .else
  INS .sfunction index, ( (index+1-INSTRUMENTS) << 1 ) |    %1 ;Instrument in Track
 .fi
.else
 JMP .sfunction index, (index-TRACKS)                  |    %0 ;Jump in Track
.fi
PAT .sfunction index, (index-PATTERNS)               +   $80 ;Pattern in Track

Note  .sfunction index, ( index << 2 )               |   %00 ;Note in Pattern
Note_ .sfunction index, ( index << 2 )               |   %10 ;Note+PatternEnd in Pattern
Ins .sfunction index, ( (index+1-INSTRUMENTS) << 1 ) |    %1 ;Instrument in Pattern
Dur .sfunction dura, ( dura * TEMPO - 1 )            +   $80 ;Duration in Pattern

.if (ABSPITCH_SUPPORT)
 ArpWF  .sfunction wf,arp, (arp << 4)                 | %0000 , wf  ;Arpeggio+WaveForm in Instrument
 ArpWF_ .sfunction wf,arp, (arp << 4)                 | %0010 , wf  ;Arpeggio+WaveForm in Instrument + End
 AbsWF  .sfunction wf,abs, (abs << 4)                 | %1000 , wf  ;Abs.Pitch+WaveForm in Instrument
 AbsWF_ .sfunction wf,abs, (abs << 4)                 | %1010 , wf  ;Abs.Pitch+WaveForm in Instrument + End
.else
 ArpWF  .sfunction wf,arp, (arp << 4)                 |  %000 , wf  ;Arpeggio+WaveForm in Instrument
 ArpWF_ .sfunction wf,arp, (arp << 4)                 |  %010 , wf  ;Arpeggio+WaveForm in Instrument + End
.fi
;.if (ARPALONE_SUPPORT)
; Arp    .sfunction arp, (arp << 4)                    | %0100       ;Arpeggio (rel.pitch) in Instrument
; Arp_   .sfunction arp, (arp << 4)                    | %0110       ;Arpeggio (rel.pitch) in Instrument + End
; SID    .sfunction reg,val, (reg << 4)                | %1100 , val ;SID-register setting in Instrument
; SID_   .sfunction reg,val, (reg << 4)                | %1110 , val ;SID-register setting in Instrument + End
;.else
 SID    .sfunction reg,val, (reg << 3)                |  %100 , val ;SID-register setting in Instrument
 SID_   .sfunction reg,val, (reg << 3)                |  %110 , val ;SID-register setting in Instrument + End
;.fi
Jmp .sfunction index, ( (index+1-INSTRUMENTS) << 1  ) |    %1       ;Jump in Instrument

;SID-register names
PW .sfunction val, SID ($03,val)
WF .sfunction val, SID ($04,val)
AD .sfunction val, SID ($05,val)
SR .sfunction val, SID ($06,val)
Vol3  .sfunction val, SID ($18-SID_CHANNEL_SPACING*2, val)
Fil3  .sfunction val, SID ($17-SID_CHANNEL_SPACING*2, val)
Ctf3  .sfunction val, SID ($16-SID_CHANNEL_SPACING*2, val)




.if (BASSNOTE_SUPPORT)

;Note pitch-values (for NoteTable) calculated by pitchgen.c, values used below
;C   C#  D   D#  E   F   F#  G   G#  A   A#  H
;04, 04, 44, 84, 84, C4, C4, 06, 46, 46, 86, C6 ;Octave1
;08, 48, 88, C8, 0A, 4A, CA, 0C, 4C, CC, 4E, 8E ;Octave2
;10, 90, 12, 92, 14, D4, 56, 18, D8, 9A, 5C, 1E ;Octave3
;20, 22, 24, 26, 68, AA, EC, 30, B2, 36, B8, 7C ;Octave4
;41, 45, 49, 4D, 51, 57, 5B, 61, 67, 6D, 73, 79 ;Octave5
;81, 89, 91, 99, A3, AD, B7, C1, CD, D9, E5, F3 ;Octave6

;Octave 1 (pitches with !!! are a bit out of tune)
c1= $04 ;( ((c6&$30)<<2)  | (c6>>5)  ) & $FE ;$0204 +$04 *
cs1=$04 ;( ((cs6&$30)<<2) | (cs6>>5) ) & $FE ;$0204 -$1A !!!
d1= $44 ;( ((d6&$30)<<2)  | (d6>>5)  ) & $FE ;$0244 +$06 *
ds1=$84 ;( ((ds6&$30)<<2) | (ds6>>5) ) & $FE ;$0244 -$1C !!!
e1= $84 ;( ((e6&$30)<<2)  | (e6>>5)  ) & $FE ;$0284 -$01 **
f1= $C4 ;( ((f6&$30)<<2)  | (f6>>5)  ) & $FE ;$0284 -$27 !!!
fs1=$C4 ;( ((fs6&$30)<<2) | (fs6>>5) ) & $FE ;$02C4 -$10
g1= $06 ;( ((g6&$30)<<2)  | (g6>>5)  ) & $FE ;$0306 +$07 *
gs1=$46 ;( ((gs6&$30)<<2) | (gs6>>5) ) & $FE ;$0306 -$26 !!!
a1= $46 ;( ((a6&$30)<<2)  | (a6>>5)  ) & $FE ;$0346 -$17
as1=$86 ;( ((as6&$30)<<2) | (as6>>5) ) & $FE ;$0386 -$0A
h1= $C6 ;( ((h6&$30)<<2)  | (h6>>5)  ) & $FE ;$03C6  0   **

;Octave 2
c2= $08 ;( ((c6&$18)<<3)  | (c6>>4)  ) & $FE ;$0408 +$08 *
cs2=$48 ;( ((cs6&$18)<<3) | (cs6>>4) ) & $FE ;$0448 +$0C
d2= $88 ;( ((d6&$18)<<3)  | (d6>>4)  ) & $FE ;$0488 +$0B
ds2=$C8 ;( ((ds6&$18)<<3) | (ds6>>4) ) & $FE ;$04C8 +$07 *
e2= $0A ;( ((e6&$18)<<3)  | (e6>>4)  ) & $FE ;$050A  0   **
f2= $4A ;( ((f6&$18)<<3)  | (f6>>4)  ) & $FE ;$054A -$0C
fs2=$CA ;( ((fs6&$18)<<3) | (fs6>>4) ) & $FE ;$058A -$1E !
g2= $0C ;( ((g6&$18)<<3)  | (g6>>4)  ) & $FE ;$060C +$0E
gs2=$4C ;( ((gs6&$18)<<3) | (gs6>>4) ) & $FE ;$064C -$0D
a2= $CC ;( ((a6&$18)<<3)  | (a6>>4)  ) & $FE ;$06CC +$12
as2=$4E ;( ((as6&$18)<<3) | (as6>>4) ) & $FE ;$070E -$12
h2= $8E ;( ((h6&$18)<<3)  | (h6>>4)  ) & $FE ;$078E +$01 **

;Octave 3
c3= $10 ;( ((c6&$C)<<4)  | (c6>>3)  ) & $FE  ;$0810 +$10
cs3=$90 ;( ((cs6&$C)<<4) | (cs6>>3) ) & $FE
d3= $12 ;( ((d6&$C)<<4)  | (d6>>3)  ) & $FE  ;$0912 +$18
ds3=$92 ;( ((ds6&$C)<<4) | (ds6>>3) ) & $FE
e3= $14 ;( ((e6&$C)<<4)  | (e6>>3)  ) & $FE  ;$0A14  0   **
f3= $D4 ;( ((f6&$C)<<4)  | (f6>>3)  ) & $FE  ;$0A94 -$19
fs3=$56 ;( ((fs6&$C)<<4) | (fs6>>3) ) & $FE
g3= $18 ;( ((g6&$C)<<4)  | (g6>>3)  ) & $FE  ;$0C18 +$1C !
gs3=$D8 ;( ((gs6&$C)<<4) | (gs6>>3) ) & $FE
a3= $9A ;( ((a6&$C)<<4)  | (a6>>3)  ) & $FE  ;$0D9A +$26 !
as3=$5C ;( ((as6&$C)<<4) | (as6>>3) ) & $FE
h3= $1E ;( ((h6&$C)<<4)  | (h6>>3)  ) & $FE  ;$0F1E +$04 **

;Octave 4
c4= $20 ;( ((c6&6)<<5)  | (c6>>2)  ) & $FE ;$1020  +$20
cs4=$22 ;( ((cs6&6)<<5) | (cs6>>2) ) & $FE
d4= $24 ;( ((d6&6)<<5)  | (d6>>2)  ) & $FE ;$1224  +$2F !
ds4=$26 ;( ((ds6&6)<<5) | (ds6>>2) ) & $FE
e4= $68 ;( ((e6&6)<<5)  | (e6>>2)  ) & $FE ;$1468  +$40 !
f4= $AA ;( ((f6&6)<<5)  | (f6>>2)  ) & $FE ;$156A  +$0F
fs4=$EC ;( ((fs6&6)<<5) | (fs6>>2) ) & $FE
g4= $30 ;( ((g6&6)<<5)  | (g6>>2)  ) & $FE ;$1830  +$37 !
gs4=$B2 ;( ((gs6&6)<<5) | (gs6>>2) ) & $FE
a4= $36 ;( ((a6&6)<<5)  | (a6>>2)  ) & $FE ;$1B36  +$4E !
as4=$B8 ;( ((as6&6)<<5) | (as6>>2) ) & $FE
h4= $7C ;( ((h6&6)<<5)  | (h6>>2)  ) & $FE ;$1E7C  +$48 !

;Octave 5 - this is not calculated as Octave1..4, but lowbyte is a duplicate of highbyte
c5= $41 ;(c6>>1) |1
cs5=$45 ;(cs6>>1)|1
d5= $49 ;(d6>>1) |1
ds5=$4D ;(ds6>>1)|1
e5= $51 ;(e6>>1) |1
f5= $57 ;($2B<<1)|1
fs5=$5B ;(fs6>>1)|1
g5= $61 ;(g6>>1) |1
gs5=$67 ;(gs6>>1)|1
a5= $6D ;(a6>>1) |1
as5=$73 ;(as6>>1)|1
h5= $79 ;(h6>>1) |1

;Octave 6 - this is not calculated as Octave1..4, but lowbyte is a duplicate of highbyte
c6= $81 ;($40<<1)|1
cs6=$89 ;($44<<1)|1
d6= $91 ;($48<<1)|1
ds6=$99 ;($4C<<1)|1
e6= $A3 ;($51<<1)|1
f6= $AD ;($55<<1)|1
fs6=$B7 ;($5B<<1)|1
g6= $C1 ;($60<<1)|1
gs6=$CD ;($66<<1)|1
a6= $D9 ;($6C<<1)|1
as6=$E5 ;($72<<1)|1
h6= $F3 ;($79<<1)|1


.else ;if No pitch lowbyte support (still many upper notes are well tuned)


;Handcrafted Pitch highbyte-values (usable as standalone values for non-bass notes

;Octave 0    1       2                3
;Note: C0   C1  G1   C2  E2  G2(A#2)  C3  D3  E3(F#3) G3 (G#3)A#3 H3
;Hz:    15   30  45  60  75  90 105  120 135 150 165 180 195 210 225
;val:  $01  $02 $03 $04 $05 $06 $07  $08 $09 $0A $0B $0C $0D $0E $0F

c0=$01 ;Octave0 C-note
cs0=c0
d0=c0
ds0=c0
e0=c0
f0=c0
fs0=c0
g0=c0
gs0=c0
a0=c0
as0=c0
h0=c0

c1=$02 ;Octave1 C-note
cs1=c1
d1=c1
ds1=c1
e1=c1
f1=c1
fs1=c1
g1=$03
gs1=g1
a1=g1
as1=g1
h1=g1

c2=$04
cs2=c2
d2=c2
ds2=c2
e2=$05
f2=e2
fs2=e2
g2=$06
gs2=g2
a2=g2
as2=$07 ;imprecise if only high-byte set
h2=as2

c3=$08
cs3=c3
d3=$09
ds3=d3
e3=$0a
f3=e3
fs3=e3
g3=$0C
gs3=$0D ;imprecise if only high-byte set
a3=g3
as3=$0E
h3=$0F

;Octave 4
;Note:  C' C#'  D' D#'  E'  f' F#'      G'    (Ab') A'    (Hb') H'
;Hz:   241 256 271 286 301 316 331 346 361 376 391 406 421 436 451 466
;val:  $10 $11 $12 $13 $14 $15 $16 $17 $18 $19 $1A $1B $1c $1D $1E $1F
c4=$10 ;$1000  0
cs4=$11;$10F3 +$0D *
d4=$12 ;$11F5 +$0B *
ds4=$13;$1307 -$07 *
e4=$14 ;$1428 -$28
f4=$15 ;$155B -$5B !
fs4=$17;$16A0 +$60 !
g4=$18 ;$17F9 +$07 *
gs4=$19;$1966 -$66 ! ;imprecise if only high-byte set ;gs4l=<(gs7<<5)
a4=$1B ;$1AE8 +$18 *
as4=$1D;$1C82 +$7E ! ;imprecise if only high-byte set ;as4l=<(as7<<5)
h4=$1E ;$1E34 -$34

;Octave 5
;Note: C'' C#'' D'' D#'' E'' F'' F#'' G'' G#'' A'' A#'' H''
;val:  $20 $22  $24 $26  $28 $2B $2d  $30 $33  $36 $39  $3D
      ;C5FREQ=481.89
c5=$20 ;$2000  0
cs5=$22;$21E7 +$19 *
d5=$24 ;$23EB +$15 *
ds5=$26;$260E -$0E *
e5=$28 ;$2851 -$51
f5=$2B ;$2AB7 +$49
fs5=$2D;$2D41 -$41
g5=$30 ;$2FF2 +$0E *
gs5=$33;$32CC +$34
a5=$36 ;$35D1 +$2F
as5=$39;$3904 -$04 *
h5=$3C ;$3C68 -$68 !

;Octave 6
      ;C5FREQ=481.89
c6=$40 ;$4000  0   *
cs6=$44;$43CE +$32
d6=$48 ;$47D6 +$2A
ds6=$4C;$4C1C -$1C *
e6=$51 ;$50A2 +$5E !
f6=$55 ;$556E -$6E !
fs6=$5B;$5A82 +$7E !
g6=$60 ;$5FE4 +$1C *
gs6=$66;$6598 +$68 !
a6=$6C ;$6BA2 +$5E !
as6=$72;$7209 -$09 *
h6=$79 ;$78D1 +$2F

;Octave 7
      ;C5FREQ=481.89
c7=$80 ;$8000  0   *
cs7=$88;$879D +$63 !
d7=$90 ;$8FAD +$53
ds7=$98;$9838 -$38
e7=$A1 ;$A145 -$45
f7=$AB ;$AADC +$24 *
fs7=$B5;$B505 -$05 *
g7=$C0 ;$BFC9 +$37
gs7=$CB;$CB30 -$30
a7=$D7 ;$D745 -$45
as7=$E4;$E412 -$12 *
h7=$F2 ;$F1A2 +$5E !


.fi

