#include <stdio.h>
#include <math.h>

#define C64_PAL_CPUFREQ 985248 //Hz

int main () {
 char* NoteNames[]= {
  "C-","C#","D-","D#","E-","F-","F#","G-","G#","A-","A#","H-",""
 };
 int i,j;
 unsigned char Octave, Note, PitchHiOnly, PitchH, PitchL, PitchByteProp,PitchByte, NoteCount, FoundBetter;
 unsigned short SIDpitch, CalcPitch,CalcPitchProp;
 double C5freq=C64_PAL_CPUFREQ/2048.0; //Hz
 double Error, ErrorProp, ErrorSum, AverageError, MaxError, MinError, MaxErrorDiff, PrevError, WeightedError, MinWeightedError;
 double MinAverageError, MinMaxError, MinMaxErrorDiff, BestC5freq, MinAvgErrorHz, MinMaxErrorHz, MinMaxErrorDiffHz, MinWeightedErrorHz;

 MinAverageError=MinMaxError=MinMaxErrorDiff=MinWeightedError=100;

 void calcFreqTable (char startoctave, char endoctave, char print) {

  if(print) printf("\nC5-note frequency: %.3fHz:",C5freq);
  NoteCount=ErrorSum=MaxError=MaxErrorDiff=PrevError=0; MinError=100;
  for(i=startoctave*12; i<endoctave*12; ++i) {

   Octave = i/12; Note = i%12;
   SIDpitch = round ( pow(256,3)/C64_PAL_CPUFREQ * C5freq * pow(2, (i-5*12) / 12.0) );
   PitchHiOnly = (unsigned char)round(SIDpitch/256.0);

   Error=100.0; j=+1;
   for (j=-4; j<=+4; ++j) {
    PitchH = (SIDpitch+j*0x40) >> 8;
    PitchL = (SIDpitch+j*0x40) & 0xFF;
    PitchByteProp = Octave>=5 ? ((PitchH<<1)|1) : ( (PitchL&0xC0) | (PitchH<<1) );
    CalcPitchProp = (PitchByteProp&1) ? ((PitchByteProp<<7)&0xFF00) | PitchByteProp
                                      : ((PitchByteProp<<7)&0x1F00) | PitchByteProp;
    ErrorProp = (CalcPitchProp-SIDpitch)*100.0/SIDpitch;
    if (fabs(ErrorProp) < fabs(Error))
    { Error=ErrorProp; PitchByte=PitchByteProp, CalcPitch=CalcPitchProp; }
   }

   if(print) {
    if(i%12==0) printf("\n");
    if(print<2) {
     if(i%3==0) printf("\n");
     printf("%s%d:%4.4X:%2.2X:%+2.1f%%,%2.2X:%4.4X:%+2.1f%%  ",
            NoteNames[Note], Octave, SIDpitch, PitchHiOnly, (PitchHiOnly*256.0-SIDpitch)*100.0/SIDpitch,
            PitchByte, CalcPitch, Error );
    }
    else printf("%2.2X, ", PitchByte);
   }

   if(fabs(Error)>fabs(MaxError)) MaxError=Error;
   if(fabs(Error)<fabs(MinError)) MinError=Error;
   ErrorSum += fabs(Error); NoteCount++;
   if(fabs(Error-PrevError)>fabs(MaxErrorDiff)) MaxErrorDiff=fabs(Error-PrevError);
   PrevError=Error;
  }

  if(print) printf("\n");
  AverageError=ErrorSum/NoteCount;
  WeightedError = AverageError/(1.1-0.68) + fabs(MaxError)/(6.61-3.45) + MaxErrorDiff/(8.14-5.73);

 }


 for(C5freq=BestC5freq=MinAvgErrorHz=MinMaxErrorHz=MinMaxErrorDiffHz=MinWeightedErrorHz=400.0; C5freq<600.0; C5freq+=0.001 ) {

  calcFreqTable(2,7,0);

  FoundBetter=0;
  if(AverageError<MinAverageError) { MinAverageError=AverageError; MinAvgErrorHz=C5freq; FoundBetter=1; }
  if(fabs(MaxError)<MinMaxError) { MinMaxError=fabs(MaxError); MinMaxErrorHz=C5freq; FoundBetter=1; }
  if(MaxErrorDiff<MinMaxErrorDiff) { MinMaxErrorDiff=MaxErrorDiff; MinMaxErrorDiffHz=C5freq; FoundBetter=1; }
  if(WeightedError<MinWeightedError) { MinWeightedError=WeightedError; MinWeightedErrorHz=C5freq; FoundBetter=1; }
  if(FoundBetter) {
   printf("NoteCount:%d, AverageError:%.2f%%, MaxError:%.2f%%, MinError:%.2f%%, MaxErrorDiff:%.2f%%, WeightedError:%.2f%% C5freq:%.3fHz\n"
          ,NoteCount,AverageError,MaxError,MinError,MaxErrorDiff,WeightedError,C5freq);
  }

 }


 printf("\nMinAverageError:%.2f%%(%.3fHz), MinMaxError:%.2f%%(%.3fHz),MinMaxErrorDiff:%.2f%%(%.3fHz), MinWeightedError%.2f%%(%.3fHz)\n"
        ,MinAverageError,MinAvgErrorHz, MinMaxError,MinMaxErrorHz, MinMaxErrorDiff,MinMaxErrorDiffHz,MinWeightedError,MinWeightedErrorHz);

 C5freq=MinAvgErrorHz; calcFreqTable(1,7,1);
 printf("\nNoteCount:%d, AverageError:%.2f%%, MaxError:%.2f%%, MaxErrorDiff:%.2f%%, WeightedError:%.2f%%\n"
        ,NoteCount,AverageError,MaxError,MaxErrorDiff,WeightedError,C5freq);
 C5freq=MinMaxErrorHz; calcFreqTable(1,7,1);
 printf("\nNoteCount:%d, AverageError:%.2f%%, MaxError:%.2f%%, MaxErrorDiff:%.2f%%, WeightedError:%.2f%%\n"
        ,NoteCount,AverageError,MaxError,MaxErrorDiff,WeightedError,C5freq);
 C5freq=MinMaxErrorDiffHz; calcFreqTable(1,7,1);
 printf("\nNoteCount:%d, AverageError:%.2f%%, MaxError:%.2f%%, MaxErrorDiff:%.2f%%, WeightedError:%.2f%%\n"
        ,NoteCount,AverageError,MaxError,MaxErrorDiff,WeightedError,C5freq);
 C5freq=MinWeightedErrorHz; calcFreqTable(1,7,2);
 printf("\nNoteCount:%d, AverageError:%.2f%%, MaxError:%.2f%%, MaxErrorDiff:%.2f%%, WeightedError:%.2f%%\n"
        ,NoteCount,AverageError,MaxError,MaxErrorDiff,WeightedError,C5freq);
 //C5freq=C64_PAL_CPUFREQ/2048.0; calcFreqTable(1);
 //printf("\nNoteCount:%d, AverageError:%.2f%%, MaxError:%.2f%%, MaxErrorDiff:%.2f%%, WeightedError:%.2f%%\n"
 //       ,NoteCount,AverageError,MaxError,MaxErrorDiff,WeightedError,C5freq);


}

