; Stack-based tiny SID music-routine by Hermit (Mihaly Horvath) - March 2022

;SID-constants
SID_CHANNELS = 3
SID_CHANNEL_SPACING = 7

;zeropage-variables
VARIABLES=$0B
VarSetTargets   = VARIABLES+0 ;These variables can be set from musicdata:
CommandIndex = VarSetTargets+0 ;Current musicdata-execution position (written by jumps/calls)
Note         = VarSetTargets+1 ;Used as base for Arpeggio indexing-call
Duration     = VarSetTargets+2 ;Note-time in frames, DurationCounter is set to this on Note-call
Instrument   = VarSetTargets+3 ;Called as a subroutine when Note-call is performed
Transpose    = VarSetTargets+4 ;Used as extra addition for Arpeggio indexing-call
DurationCounter = VARIABLES+5 ;Set on note-call, and terminates note-subroutine when expires
StackIndex      = VARIABLES+6 ;Subroutine return-address stack
VARIABLES_end = VARIABLES + SID_CHANNELS * SID_CHANNEL_SPACING
STACKS = VARIABLES_end


PLAYERCODE


init        ldx #(VARIABLES_end-VARIABLES)-1
initLoop    txa
            sta VARIABLES,x
            dex
            bpl initLoop ;X is $FF at loop-exit


play        ldx #(SID_CHANNELS-1) * SID_CHANNEL_SPACING
channelLoop

getCmdIndex ldy CommandIndex,x
nextCmd     inc CommandIndex,x
getCommand  lda MUSICDATA,y

checkReturn bne checkCall  ;$00 (Return) ?
returnSubr  dec StackIndex,x
            ldy StackIndex,x
            lda STACKS-(StackIndex-VARIABLES),y
            sta CommandIndex,x
            bne getCmdIndex ;jump

checkCall   bpl checkNote  ;$80..$FF (direct call)?
            and #$7F
callSubr    sta CommandIndex,x
            iny
            tya
            ldy StackIndex,x
            sta STACKS-(StackIndex-VARIABLES),y
            inc StackIndex,x
            bne getCmdIndex ;jump

checkNote   lsr            ;Note (indirect) call?
            bcc checkArp
callNote    sta Note,x
            lda Instrument,x
            bne callSubr ;jump

checkArp    lsr            ;Arpeggio (indexed) call?
            bcs checkSetRet
callArp     bcc nextChannel

checkSetRet lsr ;Returning to caller after SID/var-setting?
            php
            sta setSIDreg+1 ;Carry is 1 here
            sta setVariable+1
            cmp #$0B
            lda MUSICDATA+1,y
            inc CommandIndex,x
checkVarSet bcs setVariable
setSIDreg   sta $D400,x ;operand lowbyte selfmod
            .byte $2C ;skip
setVariable sta VarSetTargets,x ;operand selfmod ;may be a jump (writing CommandIndex,x)
            plp
            bcs returnSubr
            ;bcc getCmdIndex

nextChannel txa
            sbx #7
            bpl channelLoop

returnPlay  rts


.include "music.s"

