; Tiny SID music-routine by Hermit (Mihaly Horvath) - March 2022


;SID-constants
SID_CHANNELS = 3
SID_CHANNEL_SPACING = 7


;Feature-Config
.weak
 SIDINIT_SUPPORT   = 1 ;zeroes all SID-registers at init (stable pitch-low=0 and default $00 AttackDecay, no filter)
 TRANSPOSE_SUPPORT = 1 ;allows transpose-change within Track (pattern-list)
 ABSPITCH_SUPPORT  = 1 ;absolute (note/transpose-independent) pitch lookup in instrument-table (+waveform)
 TRACKINS_SUPPORT  = 1 ;allows instrument-selection from Track (pattern-list)
 PULSESWEEP_SUPPORT= 1 ;assign duration-counter to pulsewidth-lowbyte causing a slight pulse-sweep
 BASSNOTE_SUPPORT  = 1 ;use 16 bit pitch (having more notes in bass range)
 INITRETURN_SUPPORT= 1 ;have an RTS after initRoutine or let it do the 1st frame by passing through to playerRoutine
 ;ARPALONE_SUPPORT  = 0 ;1byte standalone arpeggio (relative pitch) in instrument-table (no waveform-change)
.endweak


VARIABLES = $10
TrackIndex      = VARIABLES + 0
PatternIndex    = VARIABLES + 1
InstrumentIndex = VARIABLES + 2
DurationCounter = VARIABLES + 3
Duration        = VARIABLES + 4
Instrument      = VARIABLES + 5
NoteIndex       = VARIABLES + 6 ;+ Transpose on higher bits?

.if (TRANSPOSE_SUPPORT)
 VARIABLES2 = VARIABLES + SID_CHANNELS * SID_CHANNEL_SPACING
 Transpose = VARIABLES2 + 0 ;should be merged with Note?
 VARIABLES_end = VARIABLES2 + (SID_CHANNELS-1) * SID_CHANNEL_SPACING + 1
.else
 VARIABLES_end = VARIABLES + SID_CHANNELS * SID_CHANNEL_SPACING
.fi


.logical PLAYERADDRESS

PLAYERCODE


initPlayer  ldx #(VARIABLES_end-VARIABLES)-1 ;Accumulator = 0 if init called with 1st subtune
initLoop    txa
            sta VARIABLES,x
            .if (SIDINIT_SUPPORT)
             lda #0
             sta $d400,x
            .fi
            dex
            bpl initLoop ;X is $FF at loop-exit
            .if (INITRETURN_SUPPORT)
             rts ;not really needed, but if not used, wait 1 frame from init to play in caller
            .fi

playerCall  ldx #(SID_CHANNELS-1) * SID_CHANNEL_SPACING
loadInstIdx ldy InstrumentIndex,x
countTempo  dec DurationCounter,x
            bmi loadInsPara

;Handle Tracks (pattern-lists)
loadPattIdx ldy PatternIndex,x
checkPatEnd bmi stepInPatt
loadTrckIdx ldy TrackIndex,x
stepInTrack iny
storTrckIdx sty TrackIndex,x
            lda TRACKS-1,y
            tay
            .if (TRANSPOSE_SUPPORT || TRACKINS_SUPPORT)
 checkPatt   bmi stepInPatt
 checkTrJump lsr
             tay
             bcc stepInTrack
             .if (TRANSPOSE_SUPPORT)
              .if (TRACKINS_SUPPORT)
   checkTransp lsr
               bcc setInstr
              .fi
  setTransp   sta Transpose,x
              .if (TRACKINS_SUPPORT)
               .byte $0C ;skip
              .fi
             .fi
             .if (TRACKINS_SUPPORT)
  setInstr    sta Instrument,x
             .fi
             bpl loadTrckIdx ;=jmp
            .else
             bpl stepInTrack
            .fi

;Handle Patterns
stepInPatt  iny
storPattIdx sty PatternIndex,x
loadPattDat lda PATTERNS-$81,y
chkDuration bmi setDuration
checkInstr  lsr
            bcc startNote
setInstrum  sta Instrument,x
            .byte $0C ;skip
setDuration sta Duration,x
            bne loadPattIdx ;=jmp
startNote   lsr ;Carry = Pattern-end at the same time
            bcc setNote
            sta PatternIndex,x ;trigger Track-advance
setNote     sta NoteIndex,x
            lda Duration,x
            sta DurationCounter,x
            ldy Instrument,x

;Handle Instrument-tables
loadInsPara lda INSTRUMENTS+0,y
            pha
loadInsCmd  lda INSTRUMENTS-1,y
checkInsJmp lsr
            bcc checkInsEnd
            tay
            pla
tableJump   bcs loadInsPara ;=jmp

checkArpWF  .if (ABSPITCH_SUPPORT)
             lsr
             bcs lookupPitch ;absolute-pitch?
            .fi
lookupNote  adc NoteIndex,x ;current Note, octave-shifting would be nice to expand pitch-range
            .if (TRANSPOSE_SUPPORT)
             adc Transpose,x
            .fi

lookupPitch tay
            lda NoteTable,y
            .if (BASSNOTE_SUPPORT)
 setPitchL   sta $d400,x
             lsr
             bcs setPitchH
             and #$1F
 setPitchH   sta $d401,x
            .else
 setPitchH   sta $d401,x
             .if (SIDINIT_SUPPORT==0)
             ;lda #$00 ;might be needed for good bass-pitches
 setPitchL   sta $d400,x
             .fi
            .fi

            pla
setWaveForm sta $d404,x
            .if (PULSESWEEP_SUPPORT)
             lda DurationCounter,x
             sta $d402,x
            .fi
nextChannel txa
            sbx #7
            bpl loadInstIdx
returnPlay  rts

checkInsEnd lsr
            bcs storeInsIdx
            iny
            iny ;only 1 would be needed for standalone arp (without WF)!
storeInsIdx sty InstrumentIndex,x
checkSIDset lsr
            bcc checkArpWF
;            .if (ARPALONE_SUPPORT)
; chkJustArp  lsr
;             bcc lookupNote
;            .fi
            sta setSIDreg+1
            pla
setSIDreg   sta $d400,x ;selfmod address-lowbyte
            bcs loadInsPara ;=jmp


.include "music.s"
;.include "music-example.s"
;.include "music-empty.s"

PLAYERCODE_end=*
.here

